@@ -117,7 +117,7 @@ def check_and_get_data(input_list: list, **pars: object) -> list:
                             member_suffix = '_flt.fits'
 
                         candidate_list.append(memname + member_suffix)
-            elif suffix in ['flc', 'flt']:
+            elif suffix in ['flc', 'flt', 'c0m']:
                 if lc_input_item not in candidate_list:
                     candidate_list.append(lc_input_item)
             else:
@@ -297,8 +297,12 @@ def perform_align(input_list, catalog_list, num_sources, archive=False, clobber=
     starting_dt = datetime.datetime.now()
 
     # Get default alignment parameters if not provided by the user...
-    inst = fits.getval(imglist[0], 'instrume')
-    det = fits.getval(imglist[0], 'detector')
+    hdr0 = fits.getheader(imglist[0])
+    inst = hdr0.get('instrume')
+    if inst.lower() == 'wfpc2' and 'detector' not in hdr0:
+        det = 'pc'
+    else:
+        det = hdr0.get('detector')
     apars = get_default_pars(inst, det)
     alignment_pars.update(apars)
     alignment_pars['MAX_SOURCES_PER_CHIP'] = num_sources
@@ -393,7 +397,7 @@ def perform_align(input_list, catalog_list, num_sources, archive=False, clobber=
         starting_dt = datetime.datetime.now()
         # 5: Retrieve list of astrometric sources from database
 
-        # Convert input images to tweakwcs-compatible FITSWCS objects and
+        # Convert input images to tweakwcs-compatible FITSWCSCorrector objects and
         # attach source catalogs to them.
         imglist = []
         for group_id, image in enumerate(process_list):
