@@ -54,6 +54,7 @@ import re
 import shutil
 import sys
 import traceback
+import glob
 
 import numpy as np
 from astropy.io import ascii, fits
@@ -91,7 +92,9 @@ envvar_cat_svm = {"SVM_CATALOG_SBC": 'on',
                   "SVM_CATALOG_HRC": 'on',
                   "SVM_CATALOG_WFC": 'on',
                   "SVM_CATALOG_UVIS": 'on',
-                  "SVM_CATALOG_IR": 'on'}
+                  "SVM_CATALOG_IR": 'on',
+                  "SVM_CATALOG_PC": 'on',
+                  "SVM_CATALOG_WF": 'on'}
 envvar_cat_str = "SVM_CATALOG_{}"
 
 # --------------------------------------------------------------------------------------------------------------
@@ -119,7 +122,7 @@ def create_catalog_products(total_obj_list, log_level, diagnostic_mode=False, ph
                        Specify which, if any, catalogs should be generated at all, based on detector.  This dictionary
                        needs to contain values for all instruments; namely:
 
-                       SVM_CATALOG_HRC, SVM_CATALOG_SBC, SVM_CATALOG_WFC, SVM_CATALOG_UVIS, SVM_CATALOG_IR
+                       SVM_CATALOG_HRC, SVM_CATALOG_SBC, SVM_CATALOG_WFC, SVM_CATALOG_UVIS, SVM_CATALOG_IR, SVM_CATALOG_PC
 
                        These variables can be defined with values of 'on'/'off'/'yes'/'no'/'true'/'false'.
 
@@ -512,11 +515,15 @@ def run_hap_processing(input_filename, diagnostic_mode=False, input_custom_pars_
     ramp_product_list = []
     manifest_name = ""
     found_data = False
+    # The product_list is a list of all the output products which will be put into the manifest file
+    product_list = []
+    mfile_list = []
+
     try:
-        # Parse the poller file and generate the the obs_info_dict, as well as the total detection
+        # Parse the poller file and generate the obs_info_dict, as well as the total detection
         # product lists which contain the ExposureProduct, FilterProduct, and TotalProduct objects
         # A poller file contains visit data for a single instrument.  The TotalProduct discriminant
-        # is the detector.  A TotalProduct object is comprised of FilterProducts and ExposureProducts
+        # is the detector.  A TotalProduct object is composed of FilterProducts and ExposureProducts
         # where its FilterProduct is distinguished by the filter in use, and the ExposureProduct
         # is the atomic exposure data. Note: the TotalProduct was enhanced to also be comprised
         # of an GrismExposureProduct list which is exclusive to the TotalProduct.
@@ -531,9 +538,6 @@ def run_hap_processing(input_filename, diagnostic_mode=False, input_custom_pars_
         log.info("Generate the manifest name for this visit.")
         log.info("The manifest will contain the names of all the output products.")
 
-        # The product_list is a list of all the output products which will be put into the manifest file
-        product_list = []
-
         # It is possible the total_obj_list output from the poller_utils contains only Grism/Prism
         # data and no direct images, so no further processing should be done.  If this is the case,
         # there is actually nothing to be done for the visit, except write out an empty manifest file.
@@ -555,6 +559,17 @@ def run_hap_processing(input_filename, diagnostic_mode=False, input_custom_pars_
 
         # Update all of the product objects with their associated configuration information.
         for total_item in total_obj_list:
+            for edp_file in total_item.edp_list:
+                # Pull in any manifest files from previous processing and include in product_list
+                mfiles = glob.glob(f"{edp_file.exposure_name}*_manifest.txt")
+                # If nothing is found, no nothing gets added to the product_list
+                for mfile in mfiles:
+                    with open(mfile, 'r') as finput:
+                        _ = [product_list.append(fname.strip("\n")) for fname in finput.readlines()]
+                    # Once it has been merged in with product_list for output to the manifest file,
+                    # record it's name so it can be deleted upon successful completion of this processing.
+                    mfile_list.append(mfile)
+
             log.info("Preparing configuration parameter values for total product {}".format(total_item.drizzle_filename))
 
             total_item.configobj_pars = config_utils.HapConfig(total_item,
@@ -690,6 +705,12 @@ def run_hap_processing(input_filename, diagnostic_mode=False, input_custom_pars_
             with open(manifest_name, mode='w') as catfile:
                 if total_obj_list:
                     [catfile.write("{}\n".format(name)) for name in product_list]
+        # Remove additional manifest files from previous processing now as well,
+        # Keep those manifest files, though, if running in diagnotic mode
+        if not diagnostic_mode:
+            for mfile in mfile_list:
+                if os.path.exists(mfile):
+                    os.remove(mfile)
 
         end_dt = datetime.datetime.now()
         log.info('Processing completed at {}'.format(str(end_dt)))
@@ -773,7 +794,6 @@ def run_align_to_gaia(tot_obj, log_level=logutil.logging.INFO, diagnostic_mode=F
     if align_table is None:
         headerlet_filenames = []
         try:
-            os.remove(gaia_obj.refname)
             gaia_obj.refname = None
         except (OSError, TypeError):
             pass
@@ -781,6 +801,15 @@ def run_align_to_gaia(tot_obj, log_level=logutil.logging.INFO, diagnostic_mode=F
         # Get names of all headerlet files written out to file
         headerlet_filenames = [f for f in align_table.filtered_table['headerletFile'] if f != "None"]
 
+    # Remove all reference catalogs written out to disk, if not in diagnostic mode
+    if not diagnostic_mode:
+            log.info(f"Removing reference catalogs:\n {gaia_obj.refcat_filenames}")
+            for fname in gaia_obj.refcat_filenames:
+                try:
+                    os.remove(fname)
+                except (OSError, TypeError):
+                    pass
+
     return headerlet_filenames
 # ----------------------------------------------------------------------------------------------------------------------
 
