@@ -15,10 +15,22 @@ import math
 import sys
 
 from enum import Enum
+from astropy.io import fits
 from astropy.io.fits import getheader
 from astropy.table import Table
+from astropy.stats import sigma_clipped_stats
+import numpy as np
+
+from photutils.segmentation import SourceCatalog, detect_sources
+
+from scipy import ndimage
+from skimage.transform import probabilistic_hough_line
+from skimage.feature import canny
 
 from stsci.tools import logutil
+from stsci.tools.bitmask import bitfield_to_boolean_mask
+
+from .astrometric_utils import classify_sources
 
 __taskname__ = 'analyze'
 
@@ -30,6 +42,7 @@ log = logutil.create_logger(__name__, level=logutil.logging.INFO, stream=sys.std
 __all__ = ['analyze_data', 'analyze_wrapper', 'mvm_analyze_wrapper']
 
 # Define global default keyword names for these fields
+"""
 OBSKEY = 'OBSTYPE'
 MTKEY = 'MTFLAG'
 SCNKEY = 'SCAN_TYP'
@@ -42,7 +55,29 @@ EXPKEY = 'EXPTIME'
 FGSKEY = 'FGSLOCK'
 CHINKEY = 'CHINJECT'
 DRIZKEY = 'DRIZCORR'
+"""
 
+WFPC2_KEYS = {'OBSKEY': 'IMAGETYP', 'MTKEY': 'MTFLAG', 'SCNKEY': '',
+              'FILKEY1': 'FILTNAM1', 'FILKEY2': 'FILTNAM2', 'FILKEY': 'FILTNAM1',
+              'APKEY': '', 'TARKEY': 'TARGNAME', 'EXPKEY': 'EXPTIME',
+              'FGSKEY': 'FGSLOCK', 'CHINKEY': '', 'DRIZKEY': 'DRIZCORR'}
+
+DEFAULT_KEYS = {'OBSKEY': 'OBSTYPE', 'MTKEY':' MTFLAG', 'SCNKEY': 'SCAN_TYP',
+                'FILKEY1': 'FILTER1', 'FILKEY2': 'FILTER2', 'FILKEY': 'FILTER',
+                'APKEY': 'APERTURE', 'TARKEY': 'TARGNAME', 'EXPKEY': 'EXPTIME',
+                'FGSKEY': 'FGSLOCK', 'CHINKEY': 'CHINJECT', 'DRIZKEY': 'DRIZCORR'}
+HEADER_KEYS = {'WFPC2': WFPC2_KEYS, 'DEFAULT':DEFAULT_KEYS}
+
+
+# These definitions are for ACS and WFC3
+BAD_DQ_FLAGS = [256,  # full-well saturated pixel
+                512,  # bad pixel from reference file
+                1024,  # weak charge trap
+                2048,  # A-to-D saturated pixel
+                4096  # cosmic-ray
+]
+
+MIN_LINES = 4  # Minimum number of detected lines for consideration of bad guiding
 
 # Return codes 
 class Ret_code(Enum):
@@ -53,7 +88,6 @@ class Ret_code(Enum):
     SBCHRC_DATA = 55 
     NO_VIABLE_DATA = 65
 
-
 # Annotates level to which image can be aligned according observational parameters
 # as described through FITS keywords
 class Messages(Enum):
@@ -153,6 +187,12 @@ def analyze_wrapper(input_file_list, log_level=logutil.logging.DEBUG, use_sbchrc
     mask = filtered_table["doProcess"] > 0
     filtered_table = filtered_table[mask]
 
+    # Further reduce table to only the data which is NOT affected by bad guiding
+    # This check is only to be done for MVM processing
+    if type.upper() == "MVM":
+        guide_mask = [not verify_guiding(f) for f in filtered_table["imageName"]]
+        filtered_table = filtered_table[guide_mask]
+
     good_table = None
     good_rows = []
     process_list = []
@@ -249,7 +289,7 @@ def analyze_data(input_file_list, log_level=logutil.logging.DEBUG, type=""):
     # Set logging level to user-specified level
     log.setLevel(log_level)
 
-    acs_filt_name_list = [FILKEY1, FILKEY2]
+    acs_filt_name_list = [DEFAULT_KEYS['FILKEY1'], DEFAULT_KEYS['FILKEY2']]
 
     # Interpret input filenames and adjust size of column accordingly
     max_name_length = max([len(f) for f in input_file_list])
@@ -311,22 +351,35 @@ def analyze_data(input_file_list, log_level=logutil.logging.DEBUG, type=""):
 
         # Keywords to use potentially for downstream analysis
         instrume = (header_data['INSTRUME']).upper()
-        detector = (header_data['DETECTOR']).upper()
-        subarray = header_data['SUBARRAY']
+        if instrume == 'WFPC2':
+            detector = 'PC'
+            subarray = False
+            hdr_keys = HEADER_KEYS[instrume]
+            aperture = 'PC'
+            scan_typ = 'N'  # default value of N/A
+            mtflag = 'T' if header_data[hdr_keys['MTKEY']] else 'F'
+            obstype = 'IMAGING' if (header_data[hdr_keys['OBSKEY']]).upper() == 'EXT' else 'CAL'
+
+        else:
+            hdr_keys = HEADER_KEYS['DEFAULT']
+            detector = (header_data['DETECTOR']).upper()
+            subarray = header_data['SUBARRAY']
+            aperture = (header_data[hdr_keys['APKEY']]).upper()
+            mtflag = (header_data[hdr_keys['MTKEY']]).upper()
+            obstype = (header_data[hdr_keys['OBSKEY']]).upper()
+
         date_obs = header_data['DATE-OBS']
         mjdutc = header_data['EXPSTART']
 
         # Obtain keyword values for analysis of viability
-        drizcorr = (header_data[DRIZKEY]).upper()
-        obstype = (header_data[OBSKEY]).upper()
-        mtflag = (header_data[MTKEY]).upper()
+        drizcorr = (header_data[hdr_keys['DRIZKEY']]).upper()
         scan_typ = ''
         if instrume == 'WFC3':
-            scan_typ = (header_data[SCNKEY]).upper()
+            scan_typ = (header_data[hdr_keys['SCNKEY']]).upper()
 
         sfilter = ''
         if instrume == 'WFC3':
-            sfilter = (header_data[FILKEY]).upper()
+            sfilter = (header_data[hdr_keys['FILKEY']]).upper()
         # Concatenate the two ACS filter names together with an underscore
         # If the filter name is blank, skip it
         if instrume == 'ACS':
@@ -344,14 +397,13 @@ def analyze_data(input_file_list, log_level=logutil.logging.DEBUG, type=""):
 
         # The aperture is only read for informational purposes as it is no
         # longer used for filtering input data.
-        aperture = (header_data[APKEY]).upper()
-        targname = (header_data[TARKEY]).upper()
-        exptime = header_data[EXPKEY]
-        fgslock = (header_data[FGSKEY]).upper()
+        targname = (header_data[hdr_keys['TARKEY']]).upper()
+        exptime = header_data[hdr_keys['EXPKEY']]
+        fgslock = (header_data[hdr_keys['FGSKEY']]).upper()
 
         chinject = 'NONE'
         if instrume == 'WFC3' and detector == 'UVIS':
-            chinject = (header_data[CHINKEY]).upper()
+            chinject = (header_data[hdr_keys['CHINKEY']]).upper()
 
         # Determine if the image has one of these conditions.  The routine
         # will exit processing upon the first satisfied condition.
@@ -365,44 +417,46 @@ def analyze_data(input_file_list, log_level=logutil.logging.DEBUG, type=""):
         do_process = True
         # Imaging vs spectroscopic or coronagraphic
         if obstype != 'IMAGING':
-            no_proc_key = OBSKEY
+            no_proc_key = hdr_keys['OBSKEY']
             no_proc_value = obstype
 
         # drizzling has been turned off
         elif drizcorr in ['OMIT', 'SKIPPED']:
-            no_proc_key = DRIZKEY
+            no_proc_key = hdr_keys['DRIZKEY']
             no_proc_value = drizcorr
 
         # Moving target
         elif mtflag == 'T':
-            no_proc_key = MTKEY
+            no_proc_key = hdr_keys['MTKEY']
             no_proc_value = mtflag
 
         # Bostrophidon without or with dwell (WFC3 only)
         elif any([scan_typ == 'C', scan_typ == 'D']):
-            no_proc_key = SCNKEY
+            no_proc_key = hdr_keys['SCNKEY']
             no_proc_value = scan_typ
 
         # Calibration target
         elif any(x in targname for x in ['DARK', 'TUNG', 'BIAS', 'FLAT', 'DEUT', 'EARTH-CAL']):
-            no_proc_key = TARKEY
+            no_proc_key = hdr_keys['TARKEY']
             no_proc_value = targname
 
         # Exposure time of effectively zero
         elif math.isclose(exptime, 0.0, abs_tol=1e-5):
-            no_proc_key = EXPKEY
+            no_proc_key = hdr_keys['EXPKEY']
             no_proc_value = exptime
 
         # Commanded FGS lock
         elif any(x in fgslock for x in ['GY', 'COARSE']):
-            no_proc_key = FGSKEY
+            no_proc_key = hdr_keys['FGSKEY']
             no_proc_value = fgslock
 
         # Charge injection mode
         elif chinject != 'NONE':
-            no_proc_key = CHINKEY
+            no_proc_key = hdr_keys['CHINKEY']
             no_proc_value = chinject
 
+        # Ramp filter images should not be processed for MVM products.
+        #
         # Filter name which starts with "BLOCK" for internal calibration of SBC
         # The sfilter variable may be the concatenation of two filters (F160_CLEAR)
         #
@@ -430,13 +484,18 @@ def analyze_data(input_file_list, log_level=logutil.logging.DEBUG, type=""):
                     log.warning("The Grism/Prism data {} with zero exposure time will be ignored.".format(input_file))
     
             if item.startswith(('BLOCK')):
-                no_proc_key = FILKEY
+                no_proc_key = hdr_keys['FILKEY']
                 no_proc_value = sfilter
 
+            if item.startswith(('FR')) and type.upper() == "MVM":
+                no_proc_key = hdr_keys['FILKEY']
+                no_proc_value = "Ramp data and MVM processing"
+                log.warning("The Ramp data {} with MVM processing will be ignored.".format(input_file))
+
         # If no_proc_key is set to a keyword, then this image has been found to not be viable for
         # alignment purposes.
         if no_proc_key is not None:
-            if no_proc_key != FGSKEY:
+            if no_proc_key != hdr_keys['FGSKEY']:
                 do_process = False
                 msg_type = Messages.NOPROC.value
             else:
@@ -473,3 +532,191 @@ def generate_msg(filename, msg, key, value):
     else:
         log.warning('Dataset can be aligned, but the result may be compromised.')
 
+
+# -----------------------------------------------------------------------------
+#  Line detection functions
+# -----------------------------------------------------------------------------
+
+def verify_guiding(filename, min_length=33):
+    """ Verify whether or not the input image was affected by guiding problems.
+
+    This algorithm evaluates the data from (SCI,1) to try to determine whether
+    the image was affected by guiding problems which mimic SCAN mode or GRISM data
+    with the trails in an arbitrary angle across the image.
+
+    Parameters
+    ==========
+    filename : str
+        Name of image to be evaluated
+
+    min_length : int, optional
+        Minimum length of trails (in pixels) to be detected in image.
+
+    Returns
+    ========
+    bad_guiding : bool
+        Boolean specifying whether or not the image was detected as
+        being affected by guiding problems.  Value is True if image
+        was affected.
+
+    Note: This function will only be called from analyze_wrapper if the processing
+          type is "MVM".  It is deliberately False for other processing (e.g., SVM and
+          pipeline).  However, this routine can be invoked directly for pipeline
+          processing from runastrodriz.py with the appropriate parameter setting.
+    """
+    log.info(f"Verifying that {filename} was not affected by guiding problems.")
+
+    hdu = fits.open(filename)
+    data = hdu[("SCI", 1)].data.copy()
+    scale_data = hdu[("SCI",1)].header["bunit"].endswith('/S')
+    data = np.nan_to_num(data, nan=0.0)  # just to be careful
+    if scale_data:
+        # Photutils works best in units of DN
+        scale_hdr = hdu[0].header if 'exptime' in hdu[0].header else hdu[1].header
+        scale_val = scale_hdr['exptime']
+        data *= scale_val
+    bkg_stats = sigma_clipped_stats(data, maxiters=2)
+    bkg_limit = bkg_stats[1] + bkg_stats[2]  # only need a 1-sigma detection limit here...
+    log.debug(f"bkg_limit found to be: {bkg_limit:.2f}")
+
+    data -= bkg_limit
+    imgarr = np.clip(data, 0, data.max())
+
+    # Build up a mask of all bad pixels and ignore them when looking for
+    # sources and linear features
+    dqarr = None
+    for extn in hdu:
+        if 'extname' in extn.header and extn.header['extname'] == 'DQ':
+            dqarr = hdu[extn].data.copy()
+            break
+    if dqarr is not None:
+        dqmask = bitfield_to_boolean_mask(dqarr, ignore_flags=BAD_DQ_FLAGS)
+    else:
+        dqmask = np.ones_like(data)
+    # close FITS object (just to be nice to the OS...)
+    hdu.close()
+    del hdu
+
+    # apply mask now...
+    imgarr *= dqmask
+    del dqmask  # just to clean up a little
+
+    # Determine rough number of probable sources
+    # Trying to ignore small sources (<= 4x4 pixels in size, or npixels < 17)
+    #   which are either noise peaks or head-on CRs.
+    segm = detect_sources(imgarr, 0, npixels=17)
+    log.debug(f'Detected {segm.nlabels} raw sources in {filename}')
+    if segm.nlabels < 2:
+        return False
+
+    src_cat = SourceCatalog(imgarr, segm)
+    # Remove likely cosmic-rays based on central_moments classification
+    bad_srcs = classify_sources(src_cat, 1.5)
+    # Get the label IDs for sources flagged as CRs, IDs are 1-based not 0-based
+    pt_srcs = np.where(bad_srcs == 0)[0] + 1
+    segm.remove_labels(pt_srcs)
+    src_cat = SourceCatalog(imgarr, segm)  # clean up source catalog now...
+    num_sources = len(src_cat)
+
+    # trim edges from image to avoid spurious sources
+    trim_slice=(slice(2, -2), slice(2, -2))
+
+    # Now determine whether this image was affected by guiding problems
+    bad_guiding = lines_in_image(imgarr[trim_slice], num_sources,
+                                 min_length=min_length, min_lines=MIN_LINES)
+    if bad_guiding:
+        log.warning(f"Image {filename}'s GUIDING detected as: BAD.")
+    else:
+        log.info(f"Image {filename}'s GUIDING detected as: GOOD.")
+
+    return bad_guiding
+
+
+def detect_lines(image, mask=None, min_length=17):
+    """Detect lines in the input image and return list of line parameters """
+    lines = {'num': None,
+             'startarr': None,
+             'endarr': None,
+             'angles': None,
+             'lengths': None,
+             'slopes': None}
+
+    # extract edges from image for faster line detection
+    edges = canny(image, sigma=2.5, low_threshold=0, high_threshold=25, mask=mask)
+
+    # Classic straight-line Hough transform
+    plines = probabilistic_hough_line(edges, threshold=0,
+                                     line_gap=0,
+                                     line_length=min_length)
+    if len(plines) > 0:
+        plines = np.array(plines)
+        startarr = plines[:, 0]
+        endarr = plines[:, 1]
+        rise = endarr[:, 1] - startarr[:, 1]
+        run = endarr[:, 0]-startarr[:, 0]
+        angles = np.arctan2(rise, run)
+        lines['startarr'] = startarr
+        lines['endarr'] = endarr
+        lines['angles'] = np.rad2deg(angles)
+        lines['lengths'] = np.sqrt(rise**2 + run**2)
+        lines['slopes'] = np.tan(angles + np.pi/2)
+        lines['num'] = len(plines)
+
+    return lines
+
+
+def lines_in_image(image, num_sources, mask=None, min_length=17, min_lines=4):
+    """Determine if image is dominated by linear features
+
+    Parameters
+    ----------
+    image : ndarray
+        Background-subtracted image to detect linear features in
+
+    sensitivity : float, optional
+        Increments in degrees for detecting lines in image.
+
+    Returns
+    -------
+    lines_detected : bool
+        Specifies whether or not image is dominated by linear features
+    """
+    # detect any lines in image
+    lines = detect_lines(image, mask=mask, min_length=min_length)
+    if lines['num'] is None:
+        log.debug(f"No linear features found.")
+        return False
+    else:
+        # If we have a lot of sources, we should have a lot of lines if bad
+        # Otherwise, however, min_lines is used to guard against faint fields
+        if lines['num'] < max(min_lines, num_sources/10):
+            log.debug(f"Only {lines['num']} linear features found.")
+            return False
+
+    # perform statistical analysis on detected linear features
+    # start by generating a histogram of the angles of all the lines
+    # We will ignore lines that are exactly in line with a column (+/- 90)
+    # as they are nearly always caused by CTE or saturation bleeding along the columns.
+    diff_lines = np.isclose(np.abs(lines['angles']), 90, atol=2.0)
+    angles = lines['angles'][~diff_lines]
+
+    angle_bins = np.linspace(-180., 180., 91)
+    ahist = np.histogram(angles, bins=angle_bins)
+
+    # if one peak has a more than 10% of all linear features detected,
+    # and there are more linear features than lines from saturated columns
+    # it is considered as having bad guiding.
+    max_angles = ahist[0].max()
+    alimit = max(len(angles) / 10.0, diff_lines.sum())
+
+    log.debug(f"Peak number of similar lines: {max_angles} based on a threshold of {alimit}")
+    log.debug(f"number of probable sources: {num_sources}")
+
+    #
+    # TODO:  add check against length of lines detected as well
+    #        -- Not sure what stats to use for this check...
+    #
+    # Now check to see if enough detected lines have the same (non-90 deg) orientation
+    lines_detected = (max_angles > alimit)
+    log.info(f"{max_angles} lines were similar, so linear features were detected.")
+    return lines_detected
