@@ -37,7 +37,7 @@ from astropy.table import Table, vstack, Column
 from astropy.coordinates import SkyCoord
 from astropy.io import fits as fits
 from astropy.io import ascii
-from astropy.convolution import Gaussian2DKernel
+from astropy.convolution import Gaussian2DKernel, convolve
 from astropy.stats import gaussian_fwhm_to_sigma, gaussian_sigma_to_fwhm, sigma_clipped_stats
 from astropy.visualization import SqrtStretch
 from astropy.visualization.mpl_normalize import ImageNormalize
@@ -64,7 +64,7 @@ from photutils.background import (Background2D, MMMBackground,
 from photutils.psf import (IntegratedGaussianPRF, DAOGroup,
                            IterativelySubtractedPSFPhotometry)
 
-from tweakwcs import FITSWCS
+from tweakwcs.correctors import FITSWCSCorrector
 from stwcs.distortion import utils
 from stwcs import wcsutil
 from stwcs.updatewcs import astrometry_utils
@@ -962,8 +962,8 @@ def extract_sources(img, dqmask=None, fwhm=3.0, kernel=None, photmode=None,
         dao_threshold, bkg = sigma_clipped_bkg(imgarr, sigma=4.0, nsigma=dao_nsigma)
         segment_threshold = np.ones(imgarr.shape, imgarr.dtype) * dao_threshold
 
-    segm = detect_sources(imgarr, segment_threshold, npixels=source_box,
-                          filter_kernel=kernel, connectivity=4)
+    segm = detect_sources(convolve(imgarr, kernel), segment_threshold, npixels=source_box,
+                          connectivity=4)
 
     # photutils >= 0.7: segm=None; photutils < 0.7: segm.nlabels=0
     if segm is None or segm.nlabels == 0:
@@ -993,11 +993,11 @@ def extract_sources(img, dqmask=None, fwhm=3.0, kernel=None, photmode=None,
                         kcenter - koffset: kcenter + koffset + 1].copy()
         kernel /= kernel.sum()  # normalize to total sum == 1
         log.info("Looking for crowded sources using smaller kernel with shape: {}".format(kernel.shape))
-        segm = detect_sources(imgarr, segment_threshold, npixels=source_box, filter_kernel=kernel)
+        segm = detect_sources(convolve(imgarr, kernel), segment_threshold, npixels=source_box)
 
     if deblend:
-        segm = deblend_sources(imgarr, segm, npixels=5,
-                               filter_kernel=kernel, nlevels=32,
+        segm = deblend_sources(convolve(imgarr, kernel), segm, npixels=5,
+                               nlevels=32,
                                contrast=0.01)
 
     # If classify is turned on, it should modify the segmentation map
@@ -1170,8 +1170,8 @@ def crclean_image(imgarr, segment_threshold, kernel, fwhm,
         Input array with pixels flagged as CRs set to 0.0
     """
     # Perform basic image segmentation to look for sources in image
-    segm = detect_sources(imgarr, segment_threshold, npixels=source_box,
-                          filter_kernel=kernel, connectivity=4)
+    segm = detect_sources(convolve(imgarr, kernel), segment_threshold, npixels=source_box,
+                          connectivity=4)
 
     log.debug("Detected {} sources of all types ".format(segm.nlabels))
     # photutils >= 0.7: segm=None; photutils < 0.7: segm.nlabels=0
@@ -1180,8 +1180,8 @@ def crclean_image(imgarr, segment_threshold, kernel, fwhm,
         return imgarr
 
     if deblend:
-        segm = deblend_sources(imgarr, segm, npixels=5,
-                               filter_kernel=kernel, nlevels=32,
+        segm = deblend_sources(convolve(imgarr, kernel), segm, npixels=5,
+                               nlevels=32,
                                contrast=0.01)
 
     # Measure segment sources to compute centralized moments for each source
@@ -1820,7 +1820,7 @@ def find_hist2d_offset(filename, reference, refwcs=None, refnames=['ra', 'dec'],
 #
 ##############################
 def build_wcscat(image, group_id, source_catalog):
-    """ Return a list of `~tweakwcs.tpwcs.FITSWCS` objects for all chips in an image.
+    """ Return a list of `~tweakwcs.correctors.FITSWCSCorrector` objects for all chips in an image.
 
     Parameters
     ----------
@@ -1834,15 +1834,15 @@ def build_wcscat(image, group_id, source_catalog):
 
     source_catalog : dict
         If provided, these catalogs will be attached as `catalog`
-        entries in each chip's ``FITSWCS`` object.  It should be provided as a
+        entries in each chip's ``FITSWCSCorrector`` object.  It should be provided as a
         dict of astropy Tables identified by chip number with
         each table containing sources from image extension ``('sci', chip)`` as
         generated by `generate_source_catalog()`.
 
     Returns
     -------
-    wcs_catalogs : list of `~tweakwcs.tpwcs.FITSWCS`
-        List of `~tweakwcs.tpwcs.FITSWCS` objects defined for all chips in input image.
+    wcs_catalogs : list of `~tweakwcs.correctors.FITSWCSCorrector`
+        List of `~tweakwcs.correctors.FITSWCSCorrector` objects defined for all chips in input image.
 
     """
     open_file = False
@@ -1880,7 +1880,7 @@ def build_wcscat(image, group_id, source_catalog):
             imcat.rename_column('xcentroid', 'x')
             imcat.rename_column('ycentroid', 'y')
 
-        wcscat = FITSWCS(
+        wcscat = FITSWCSCorrector(
             w,
             meta={
                 'chip': chip,
