@@ -37,7 +37,7 @@ SKYCELL_OVERLAP = 256
 SUPPORTED_SCALES = {'fine': 0.04, 'coarse': 0.12}  # arcseconds/pixel
 
 
-def get_sky_cells(visit_input, input_path=None, scale=None, cell_size=None):
+def get_sky_cells(visit_input, input_path=None, scale=None, cell_size=None, diagnostic_mode=False):
     """Return all sky cells that overlap the exposures in the input.
 
     Parameters
@@ -142,8 +142,12 @@ def get_sky_cells(visit_input, input_path=None, scale=None, cell_size=None):
             footprint = SkyFootprint(meta_wcs)
             footprint.build([expname])
 
+            if diagnostic_mode:
+                wcshdr = meta_wcs.to_header()
+                fits.PrimaryHDU(data=footprint.total_mask.astype(np.int16), header=wcshdr).writeto(f"{visit_id}_{expname.split('_')[-2]}_footprint.fits", overwrite=True)
+
             # Use this footprint to identify overlapping sky cells
-            visit_cells = sky_grid.get_sky_cells(footprint)
+            visit_cells = sky_grid.get_sky_cells(footprint, diagnostic_mode=diagnostic_mode)
             print('Exposure {} from visit {} overlapped SkyCells:\n{}'.format(expname, visit_id, visit_cells))
 
             for scell in visit_cells:
@@ -333,8 +337,8 @@ class SkyFootprint(object):
                 meta_x = np.clip(meta_x, 0, self.meta_wcs.array_shape[1] - 1)
 
                 # define subarray spanned by this chip on the SkyCell
-                scell_slice = [slice(meta_y.min(), meta_y.max()),
-                               slice(meta_x.min(), meta_x.max())]
+                scell_slice = (slice(meta_y.min(), meta_y.max()),
+                               slice(meta_x.min(), meta_x.max()))
                 scell_ltm = [meta_x.min(), meta_y.min()]
                 # Reset range of pixels to be relative to starting pixel position in SkyCell
                 meta_x -= scell_ltm[0]
@@ -525,7 +529,7 @@ class SkyFootprint(object):
         if len(self.members) == 0:
             print("Please add exposures before looking for corners...")
             return
-
+        label_border = 10
         # Insure footprint has been determined
         if self.footprint_member != member:
             self.find_footprint(member=member)
@@ -548,34 +552,34 @@ class SkyFootprint(object):
             ordered_xy = []
             ordered_edges = []
             sky_corners = []
-
             for label, mask_slice in enumerate(slices):
                 label += 1
+                # Need to guarantee the slice ALWAYS has a border of non-assigned pixels
+                label_shape = (mask_slice[0].stop - mask_slice[0].start + (label_border * 2),
+                               mask_slice[1].stop - mask_slice[1].start + (label_border * 2))
+                label_mask = np.zeros(label_shape, sclabels.dtype)
                 # get slice with just the region/label of interest
-                label_mask = sclabels[mask_slice].copy()
+                label_mask[label_border:-1*label_border, label_border:-1*label_border] = sclabels[mask_slice].copy()
                 # make sure no pixels from other regions are present in this mask
                 label_mask[label_mask != label] = 0
                 # reset label to be a binary mask only
                 label_mask[label_mask == label] = 1000
-                # insure there is a border all around the region
-                # THIS IS CRITICAL to being able to identify corners correctly in slice
-                label_mask = ndimage.binary_erosion(label_mask)
                 print('extracting corners for region {} in slice {}'.format(label, mask_slice))
                 # Perform corner detection on each region/chip separately.
                 mask_corners = corner_peaks(corner_harris(label_mask),
-                                       min_distance=1,
+                                       min_distance=3,
                                        threshold_rel=0.2)
                 xy_corners = mask_corners * 0.
                 xy_corners[:, 0] = mask_corners[:, 1]
                 xy_corners[:, 1] = mask_corners[:, 0]
                 # shift corner positions to full array positions
-                xy_corners += (mask_slice[1].start, mask_slice[0].start)
+                xy_corners += (mask_slice[1].start-label_border, mask_slice[0].start-label_border)
 
                 # Create a mask from the total footprint consisting solely of the
                 # pixels at the outer edge, ordered in clockwise fashion.
                 #
                 # get list of (X,Y) coordinates of all edges from each separate 'region' or chip
-                edge_pixels = trace_polygon(label_mask, mask_slice)
+                edge_pixels = trace_polygon(label_mask > 0, mask_slice)
 
                 # use the ordering of the traced edge pixels to order the corners in the same way
                 cordist = distance.cdist(xy_corners, edge_pixels)  # returns distances for each corner position
@@ -756,13 +760,14 @@ class GridDefs(object):
         else:
             self.projection_cells = [ProjectionCell(index=i, scale=self.scale) for i in id]
 
-    def get_sky_cells(self, skyfootprint, member='total'):
+    def get_sky_cells(self, skyfootprint, member='total', diagnostic_mode=False):
 
         self.get_projection_cells(skyfootprint)
 
         # Find sky cells from identified projection cell(s) that overlap footprint
         sky_cells = {}
         for pcell in self.projection_cells:
+            pcell.diagnostic_mode = diagnostic_mode
             sky_cells.update(pcell.find_sky_cells(skyfootprint,
                                                  nxy=self.sc_nxy,
                                                  overlap=self.sc_overlap))
@@ -872,6 +877,7 @@ class ProjectionCell(object):
         self.mask = None
         # Generate WCS for projection cell
         self._build_wcs()
+        self.diagnostic_mode = False
 
     def _from_index(self, id):
         grid_defs = GridDefs()
@@ -957,6 +963,13 @@ class ProjectionCell(object):
             for yi in range(mosaic_yr[0], mosaic_yr[1]):
                 skycell = SkyCell(x=xi, y=yi, projection_cell=self)
 
+                if self.diagnostic_mode:
+                    mask_fits_name = f'skycell_{skycell.sky_cell_id}_mask.fits'
+                    if not os.path.exists(mask_fits_name):
+                        skycell.build_mask()
+                        fits.PrimaryHDU(data=skycell.mask.astype(np.int16), header=skycell.wcs.to_header()).writeto(
+                            mask_fits_name, overwrite=True)
+
                 sc_overlap = self.compute_overlap(skycell, mosaic_ra, mosaic_dec)
 
                 print("    Checking SkyCell {},{} for overlap: {}".format(xi, yi, sc_overlap))
@@ -1265,7 +1278,6 @@ def _poly_trace(input_mask, box_size=3):
     mask = np.zeros((input_mask.shape[0] + (border * 2), input_mask.shape[1] + (border * 2)),
                     dtype=input_mask.dtype)
     mask[border:-border, border:-border] = input_mask.copy()
-
     for x in range(mask.shape[1]):
         pts = np.where(mask[:, x] == 1)[0]
         if len(pts) > 0:
