@@ -20,9 +20,10 @@ All the stats are sigma clipped.  These are calculated by the
 functions in aperture_stats_tbl.
 
 .. note::
-    Currently, the background computations will fully include a pixel that has ANY overlap with the background aperture
-    (the annulus). This is to simplify the computation of the median, as a weighted median is nontrivial, and slower.
-    Copied from https://grit.stsci.edu/HLA/software/blob/master/HLApipeline/HLApipe/scripts/photometry_tools.py
+    Currently, the background computations will fully include a pixel that has ANY overlap with the
+    background aperture (the annulus). This is to simplify the computation of the median, as a weighted
+    median is nontrivial, and slower. Copied from https://grit.stsci.edu/HLA/software/blob/master/HLApipeline/HLApipe/scripts/photometry_tools.py
+
 Authors
 -------
     - Varun Bajaj, January 2018
@@ -58,10 +59,11 @@ Classes and Functions
 ---------------------
 """
 import numpy as np
+import math
 from astropy.table import Table
+from drizzlepac.haputils import constants
 from drizzlepac.haputils.background_median import aperture_stats_tbl
-from photutils import aperture_photometry
-
+from photutils.aperture import aperture_photometry
 
 def iraf_style_photometry(phot_apertures, bg_apertures, data, photflam, photplam, error_array=None,
                           bg_method='mode', epadu=1.0):
@@ -112,9 +114,9 @@ def iraf_style_photometry(phot_apertures, bg_apertures, data, photflam, photplam
     final_stacked = np.stack([x, y, phot["id"].data], axis=1)
     # n_aper = 0
     name_list = 'Flux', 'FluxErr', 'Mag', 'MagErr'
-    for aper_string  in ['Ap1', 'Ap2']:
+    for aper_string in ['Ap1', 'Ap2']:
         for col_name in name_list:
-            names.append("{}{}".format(col_name,aper_string))
+            names.append("{}{}".format(col_name, aper_string))
 
     # for item in list(phot.keys()):
     #     if item.startswith("aperture_sum_") and not item.startswith("aperture_sum_err_"):
@@ -140,11 +142,15 @@ def iraf_style_photometry(phot_apertures, bg_apertures, data, photflam, photplam
         else:
             flux_error = compute_phot_error(flux, bg_phot, bg_method, ap_area, epadu)
 
+        # When photflam is 0.0, mag is set to -9999.0.
         mag = convert_flux_to_abmag(flux, photflam, photplam)
 
         # NOTE: Magnitude error calculation comes from computing d(ABMAG)/d(flux).
         # See https://iraf.net/forum/viewtopic.php?showtopic=83932 for details.
-        mag_err = 1.0857 * flux_error / flux
+        if math.isclose(photflam, 0.0, abs_tol=constants.TOLERANCE):
+            mag_err = mag
+        else:
+            mag_err = 1.0857 * flux_error / flux
 
         # Build the final data table
         stacked = np.stack([flux, flux_error, mag, mag_err], axis=1)
@@ -191,7 +197,7 @@ def compute_phot_error(flux_variance, bg_phot, bg_method, ap_area, epadu=1.0):
         an array of flux errors
     """
 
-    bg_variance_terms = (ap_area * bg_phot['aperture_std'] ** 2.) * (1. + ap_area/bg_phot['aperture_area'])
+    bg_variance_terms = (ap_area * bg_phot['aperture_std'] ** 2.) * (1. + ap_area / bg_phot['aperture_area'])
     variance = flux_variance / epadu + bg_variance_terms
     flux_error = variance ** .5
     return flux_error
@@ -217,13 +223,19 @@ def convert_flux_to_abmag(in_flux, photflam, photplam):
         input flux values converted to ABMAG
     """
 
-    # convert flux from units of electrons/second to ergs/cm2/angstrom/second
-    f_lambda = in_flux * photflam
+    # Avoid taking the log10 of zero
+    if math.isclose(photflam, 0.0, abs_tol=constants.TOLERANCE):
+        # Create a column of data from an existing column of data
+        # and set each value = -9999.0
+        abmag = constants.FLAG + in_flux * 0.0
+    else:
+        # convert flux from units of electrons/second to ergs/cm2/angstrom/second
+        f_lambda = in_flux * photflam
 
-    # Convert f_lambda to STMAG
-    stmag = -2.5 * np.log10(f_lambda) - 21.10
+        # Convert f_lambda to STMAG
+        stmag = -2.5 * np.log10(f_lambda) - 21.10
 
-    # Convert STMAG to ABMAG
-    abmag =  stmag - 5.0 * np.log10(photplam) + 18.6921
+        # Convert STMAG to ABMAG
+        abmag = stmag - 5.0 * np.log10(photplam) + 18.6921
 
-    return abmag
\ No newline at end of file
+    return abmag
