@@ -16,7 +16,7 @@ from astropy.io import fits
 from . import util
 from . import wcs_functions
 from . import buildmask
-from .version import *
+from . import __version__
 
 __all__ = ['baseImageObject', 'imageObject', 'WCSObject']
 
@@ -511,10 +511,23 @@ class baseImageObject:
     def getGain(self, exten):
         return self._image[exten]._gain
 
-    def getflat(self, chip):
+    def getflat(self, chip, flat_file=None, flat_ext=None):
         """
         Method for retrieving a detector's flat field.
 
+        Parameters
+        ----------
+        chip : int
+            Chip number. Same as FITS ``EXTVER``.
+
+        flat_file : str, None
+            Flat field file name. If not specified, it will be determined
+            automatically from image header.
+
+        flat_ext : str, None
+            Flat field extension name (same as FITS ``EXTNAME``). Specifies
+            extension name containing flat field data.
+
         Returns
         -------
         flat: array
@@ -522,6 +535,8 @@ class baseImageObject:
             **units of electrons**.
 
         """
+        if flat_ext is None:
+            flat_ext = self.scienceExt
         sci_chip = self._image[self.scienceExt, chip]
         # The keyword for ACS flat fields in the primary header of the flt
         # file is pfltfile.  This flat file is already in the required
@@ -529,12 +544,12 @@ class baseImageObject:
 
         # The use of fileutil.osfn interprets any environment variable, such as
         # jref$, used in the specification of the reference filename
-        filename = fileutil.osfn(self._image["PRIMARY"].header[self.flatkey])
+        if flat_file is None:
+            flat_file = fileutil.osfn(self._image["PRIMARY"].header[self.flatkey])
         hdulist = None
         try:
-            hdulist = fileutil.openImage(filename, mode='readonly',
-                                         memmap=False)
-            data = hdulist[(self.scienceExt, chip)].data
+            hdulist = fileutil.openImage(flat_file, mode='readonly', memmap=False)
+            data = hdulist[(flat_ext, chip)].data
 
             if data.shape[0] != sci_chip.image_shape[0]:
                 ltv2 = int(np.round(sci_chip.ltv2))
@@ -552,7 +567,7 @@ class baseImageObject:
 
         except FileNotFoundError:
             flat = np.ones(sci_chip.image_shape, dtype=sci_chip.image_dtype)
-            log.warning("Cannot find flat field file '{}'".format(filename))
+            log.warning("Cannot find flat field file '{}'".format(flat_file))
             log.warning("Treating flatfield as a constant value of '1'.")
 
         finally:
@@ -1071,7 +1086,7 @@ class imageObject(baseImageObject):
                     sci_chip.dqname = sci_chip.dqfile +'['+sci_chip.dq_extn+','+str(chip)+']'
 
                 # build up HSTWCS object for each chip, which will be necessary for drizzling operations
-                sci_chip.wcs=wcs_functions.get_hstwcs(self._filename,self._image,sci_chip.extnum)
+                sci_chip.wcs=wcs_functions.get_hstwcs(self._filename, sci_chip.extnum)
                 sci_chip.detnum,sci_chip.binned = util.get_detnum(sci_chip.wcs,self._filename,chip)
                 sci_chip.wcslin_pscale = 1.0
 
