@@ -109,6 +109,7 @@ from drizzlepac import processInput  # used for creating new ASNs for _flc input
 
 from drizzlepac import align
 from drizzlepac import resetbits
+from drizzlepac.haputils import analyze
 from drizzlepac.haputils import astrometric_utils as amutils
 from drizzlepac.haputils import cell_utils
 from drizzlepac.haputils import processing_utils
@@ -117,6 +118,10 @@ from drizzlepac import mdzhandler
 from drizzlepac import updatehdr
 from drizzlepac.haputils import quality_analysis as qa
 from drizzlepac import wcs_functions
+# for WFPC2 support
+from drizzlepac.haputils import config_utils
+from drizzlepac import wfpc2Data
+
 from . import __version__
 
 
@@ -152,7 +157,9 @@ focus_pars = {"WFC3/IR": {'sigma': 2.0, 'good_bits': 512},
               "ACS/WFC": {'sigma': 1.5, 'good_bits': 1360},
               "ACS/SBC": {'sigma': 2.0, 'good_bits': 0},
               "ACS/HRC": {'sigma': 1.5, 'good_bits': 1360},
-              "WFPC2/PC": {'sigma': 1.5, 'good_bits': 1360}}
+              "WFPC2/PC": {'sigma': 1.5, 'good_bits': 1360},
+              "WFPC2/WF": {'sigma': 1.5, 'good_bits': 1360}}
+
 sub_dirs = ['OrIg_files', 'pipeline-default']
 valid_alignment_modes = ['apriori', 'aposteriori', 'default-pipeline']
 gsc240_date = '2017-10-01'
@@ -181,7 +188,8 @@ wcs_preference = ['IDC_?????????-FIT_REL_GAIA*', 'IDC_?????????-FIT_IMG_GAIA*',
 
 # Primary user interface
 def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
-            headerlets=True, align_to_gaia=True, force_alignment=False, debug=False):
+            headerlets=True, align_to_gaia=True, force_alignment=False,
+            do_verify_guiding=True, debug=False, make_manifest=False):
     """ Run astrodrizzle on input file/ASN table
         using default values for astrodrizzle parameters.
     """
@@ -194,6 +202,7 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
     trlmsg += "    numpy version {}".format(np.__version__)
     pipeline_pars = PIPELINE_PARS.copy()
     _verify = True  # Switch to control whether to verify alignment or not
+    manifest_list = []
 
     # interpret envvar variable, if specified
     if envvar_compute_name in os.environ:
@@ -264,8 +273,20 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
 
     # Identify WFPC2 inputs to account for differences in WFPC2 inputs
     infile_inst = fits.getval(inFilename, 'instrume')
-    infile_det = fits.getval(inFilename, 'detector')
     wfpc2_input = infile_inst == 'WFPC2'
+    if not wfpc2_input:
+        raw_suffix = '_raw.fits'
+        goodpix_name = 'NGOODPIX'
+    else:
+        # Convert input c0m file into compatible flt file
+        if 'd0m' in inFilename:
+            inFilename = inFilename.replace('d0m', 'c0m')
+        # This returns the name of the _flt.fits file that was created
+        inFilename = wfpc2Data.wfpc2_to_flt(inFilename)
+        raw_suffix = '_d0m.fits'
+        goodpix_name = 'GPIXELS'
+
+    infile_det = fits.getval(inFilename, 'detector')
     cal_ext = None
 
     # Check input file to see if [DRIZ/DITH]CORR is set to PERFORM
@@ -290,12 +311,9 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
     else:
         # Check to see if input is a _RAW file
         # If it is, strip off the _raw.fits extension...
-        _indx = inFilename.find('_raw')
+        _indx = inFilename.find(raw_suffix)
         if _indx < 0: _indx = len(inFilename)
         # ... and build the CALXXX product rootname.
-        if wfpc2_input:
-            # force code to define _c0m file as calibrated product to be used
-            cal_ext = ['_c0m.fits']
         _mname = fileutil.buildRootname(inFilename[:_indx], ext=cal_ext)
 
         _cal_prodname = inFilename[:_indx]
@@ -326,6 +344,7 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
 
     _trlfile = _trlroot + '.tra'
     _alignlog = _trlroot + '_align.log'
+    _manifest_filename = _trlroot + '_manifest.txt'
     _calfiles_flc = []
 
     # Write message out to temp file and append it to full trailer file
@@ -341,7 +360,7 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
         dcorr = 'PERFORM'
     else:
         if _mname:
-            _fimg = fits.open(fileutil.buildRootname(_mname, ext=['_raw.fits']), memmap=False)
+            _fimg = fits.open(fileutil.buildRootname(_mname, ext=[raw_suffix]), memmap=False)
             _phdr = _fimg['PRIMARY'].header
             if dkey in _phdr:
                 dcorr = _phdr[dkey]
@@ -404,6 +423,26 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
     for f in _calfiles+_calfiles_flc:
         processing_utils.compute_sregion(f)
 
+    # Check to see whether or not guide star failure affected these observations
+    # They would show up as images all sources streaked as if taken in SCAN mode or with a GRISM
+    # 
+    # Note: This functionality is intentionally turned off for pipeline processing at this time.
+    # However, a user may wish to invoke this functionality which is controlled by the
+    # parameter "do_verify_guiding".
+    if do_verify_guiding:
+        for fltimg in _calfiles:
+            flcimg = f.replace('_flt.fits', '_flc.fits')
+            if os.path.exists(flcimg):
+                fltimg = flcimg
+            # We want to use the FLC image, if possible, to avoid any
+            # possible detection of CTE tails as false guide-star trailing lines
+            bad_guiding = analyze.verify_guiding(fltimg)
+            if bad_guiding:
+                # Remove the affected image(s) from further processing
+                _calfiles.remove(fltimg)
+                if os.path.exists(flcimg):
+                    _calfiles_flc.remove(flcimg)
+
     if dcorr == 'PERFORM':
 
         """
@@ -440,10 +479,15 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
         """
         inst_mode = "{}/{}".format(infile_inst, infile_det)
         _good_images = [f for f in _calfiles if fits.getval(f, 'exptime') > 0.]
-        _good_images = [f for f in _good_images if fits.getval(f, 'ngoodpix', ext=("SCI", 1)) > 0.]
+        _good_images = [f for f in _good_images if fits.getval(f, goodpix_name, ext=("SCI", 1)) > 0.]
         if len(_good_images) == 0:
             _good_images = _calfiles
-        adriz_pars = mdzhandler.getMdriztabParameters(_good_images)
+        if not wfpc2_input:
+            adriz_pars = mdzhandler.getMdriztabParameters(_good_images)
+        else:
+            default_pars = config_utils.get_wfpc2_pars(_good_images)
+            adriz_pars = default_pars['astrodrizzle'].outpars
+
         adriz_pars.update(pipeline_pars)
         adriz_pars['mdriztab'] = False
         adriz_pars['final_fillval'] = "INDEF"
@@ -474,11 +518,11 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
         if align_with_apriori or force_alignment or align_to_gaia:
             # Generate initial default products and perform verification
             align_dicts, align_table = verify_alignment(_inlist,
-                                             _calfiles, _calfiles_flc,
-                                             _trlfile,
-                                             tmpdir=None, debug=debug,
-                                             force_alignment=force_alignment,
-                                             find_crs=True, **adriz_pars)
+                                                        _calfiles, _calfiles_flc,
+                                                        _trlfile,
+                                                        tmpdir=None, debug=debug,
+                                                        force_alignment=force_alignment,
+                                                        find_crs=True, **adriz_pars)
 
         if align_with_apriori:
             _trlmsg = _timestamp('Starting alignment with a priori solutions')
@@ -492,6 +536,9 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
             # have a priori solutions as extensions
             # FIX: This should probably only be done in the apriori sub-directory!
             updatewcs.updatewcs(_calfiles)
+            for _file in _calfiles:
+                confirm_aposteriori_hdrlets(_file, logfile=_trlfile)
+
             _trlmsg += "Adding apriori WCS solutions to {}\n".format(_calfiles)
             _trlmsg += verify_gaia_wcsnames(_calfiles) + '\n'
             _wnames_calfiles = [(c, fits.getval(c, 'wcsname', ext=1)) for c in _calfiles]
@@ -501,6 +548,9 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
             if _calfiles_flc:
                 _trlmsg += "Adding apriori WCS solutions to {}\n".format(_calfiles_flc)
                 updatewcs.updatewcs(_calfiles_flc)
+                for _file in _calfiles_flc:
+                    confirm_aposteriori_hdrlets(_file, logfile=_trlfile)
+
                 _trlmsg += verify_gaia_wcsnames(_calfiles_flc) + '\n'
 
             try:
@@ -595,6 +645,9 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
         pipeline_pars['in_memory'] = inmemory
         pipeline_pars['clean'] = True
         pipeline_pars['num_cores'] = num_cores
+        if wfpc2_input:
+            pipeline_pars.update(adriz_pars)
+            pipeline_pars['mdriztab'] = False
 
         drz_products, asn_dicts, diff_dicts = run_driz(_inlist, _trlfile, _calfiles,
                                              verify_alignment=False,
@@ -612,7 +665,13 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
         # names
         for drz_product in drz_products:
             _plower = drz_product.lower()
-            if drz_product != _plower: os.rename(drz_product, _plower)
+            if drz_product != _plower:
+                os.rename(drz_product, _plower)
+                drz_output = _plower
+            else:
+                drz_output = drz_product
+            # keep track of output files
+            manifest_list.append(drz_output)
 
     else:
         # Create default trailer file messages when astrodrizzle is not
@@ -629,7 +688,8 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
 
     # If we created a new ASN table, we need to remove it
     if _new_asn is not None:
-        for _name in _new_asn: fileutil.removeFile(_name)
+        for _name in _new_asn:
+            fileutil.removeFile(_name)
 
     # Insure all input FLC/FLT files have updated WCSTYPE* keywords
     for fname in _calfiles + _calfiles_flc:
@@ -649,6 +709,8 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
                         scihdr[wname] = wval
 
                     scihdr[wtype] = updatehdr.interpret_wcsname_type(wval)
+    # Keep track of input files that have been updated
+    manifest_list.extend(_calfiles + _calfiles_flc)
 
     # If headerlets have already been written out by alignment code,
     # do NOT write out this version of the headerlets
@@ -674,6 +736,8 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
                                               attach=False,
                                               clobber=True,
                                               logging=False)
+                    # Keep track of headerlet files written out to disk
+                    manifest_list.append(hdrname)
                 except ValueError:
                     hlet_msg += _timestamp("SKIPPED: Headerlet not created for %s \n" % fname)
                     # update trailer file to log creation of headerlet files
@@ -682,9 +746,21 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
         ftrl.write(hlet_msg)
         ftrl.close()
 
+    # add trailer file to list of output products
+    manifest_list.append(_trlfile)
+
     # Remove secondary log files for good...
     logging.shutdown()
 
+    # write out manifest file, if requested
+    if make_manifest:
+        if os.path.exists(_manifest_filename):
+            os.remove(_manifest_filename)
+        with open(_manifest_filename, 'w') as fout:
+            _ = [fout.write(f"{fname}\n") for fname in manifest_list]
+        print(f"Created manifest file: {_manifest_filename}")
+
+    # delete log files generated by alignment code
     for _olog in [_alignlog]:
         if os.path.exists(_olog):
             os.remove(_olog)
@@ -701,10 +777,16 @@ def process(inFile, force=False, newpath=None, num_cores=None, inmemory=True,
         print([ofile.path for ofile in Process().open_files()])
 
     if not debug:
+        # clean up any temporary ref files created for WFPC2 data
+        if wfpc2_input:
+            d2im_file = f"{inFilename.split('_')[0]}_flt_d2im.fits"
+            if os.path.exists(d2im_file):
+                os.remove(d2im_file)
         try:
             # Remove all temp sub-directories now that we are done
             for sd in sub_dirs:
-                if os.path.exists(sd): rmtree2(sd)
+                if os.path.exists(sd):
+                    rmtree2(sd)
         except Exception:
             # If we are unable to remove any of these sub-directories,
             # leave them for the user or calling routine/pipeline to clean up.
@@ -914,8 +996,12 @@ def verify_alignment(inlist, calfiles, calfiles_flc, trlfile,
 
         alignfiles = calfiles_flc if calfiles_flc else calfiles
         align_update_files = calfiles if calfiles_flc else None
-        inst = fits.getval(alignfiles[0], 'instrume').lower()
-        det = fits.getval(alignfiles[0], 'detector').lower()
+        hdr0 = fits.getheader(alignfiles[0])
+        inst = hdr0.get('instrume').lower()
+        if inst == 'wfpc2' and 'detector' not in hdr0:
+            det = 'pc'
+        else:
+            det = hdr0.get('detector').lower()
 
         if find_crs:
             trlmsg = _timestamp("Resetting CRs ")
@@ -1260,6 +1346,7 @@ def verify_gaia_wcsnames(filenames, catalog_name='GSC240', catalog_date=gsc240_d
                                 fhdu[('sci', extn + 1)].header['idcscale'] = fhdu_idscale
     return msg
 
+
 def restore_pipeline_default(files):
     """Restore pipeline-default IDC_* WCS as PRIMARY WCS in all input files"""
     print("Restoring pipeline-default WCS as PRIMARY WCS... using updatewcs.")
@@ -1678,6 +1765,44 @@ def update_active_wcs(filename, wcsname, logfile=None):
     if logfile:
         _updateTrlFile(logfile, update_msg)
 
+
+def confirm_aposteriori_hdrlets(filename, logfile=None):
+    """Confirm that all the a posteriori headerlets are valid, and remove any that are invalid."""
+    update_msg = ""
+    num_sci_ext, extname = util.count_sci_extensions(filename)
+    extname_list = [(extname, x + 1) for x in range(num_sci_ext)]
+
+    hdu = fits.open(filename, mode='update')
+    # Get a list of all HDRLET extensions
+    hdrlet_extensions = wcsutil.headerlet.get_extname_extver_list(hdu, sciext='HDRLET')
+    hdrlet_wcsnames = wcsutil.headerlet.get_headerlet_kw_names(hdu, 'wcsname')
+
+
+    invalid_extns = []
+    for wname, extname in zip(hdrlet_wcsnames, hdrlet_extensions):
+        # Only evaluate a posteriori headerlets -- those with '-FIT' in WCSNAME
+        if '-FIT' in wname:
+            hdrlet = hdu[extname].headerlet
+            valid_ctype = '-SIP' in hdrlet[1].header['ctype1']
+            valid_dist_kws = 'A_ORDER' in hdrlet[1].header
+            if not valid_ctype or not valid_dist_kws:
+                key = wcsutil.altwcs.getKeyFromName(hdu['SCI', 1].header, wname)
+                invalid_extns.append({'extname':extname, 'wcsname':wname, 'key':key})
+
+    hdu.close()
+    # If any invalid headerlets are found, remove them from
+    if invalid_extns:
+        for extn in reversed(invalid_extns):
+            wcsutil.headerlet.delete_headerlet(filename, hdrext=extn['extname'])
+            # also remove this solution from SCI headers
+            if extn['key']:
+                wcsutil.altwcs.deleteWCS(filename, extname_list, wcskey=extn['key'])
+            update_msg += "Delete duplicate headerlet extension {} in filename {}.\n".format(extn, filename)
+
+    if logfile:
+        _updateTrlFile(logfile, update_msg)
+
+
 def _update_wcs_fit_keywords(fltfile, flcfile):
     """Update the header of the FLT file with the a posteriori fit results"""
     fit_kws_sci = [('RMS_RA', -1.0), ('RMS_DEC', -1.0),
@@ -1721,6 +1846,7 @@ def _update_wcs_fit_keywords(fltfile, flcfile):
     del hdulist
     del hdulist_flc
 
+
 def _lowerAsn(asnfile):
     """ Create a copy of the original asn file and change
         the case of all members to lower-case.
@@ -1741,6 +1867,7 @@ def _lowerAsn(asnfile):
 
     return _new_asn
 
+
 def _updateTrlFile(trlfile, trl_lines):
     tmptrl = trlfile.replace('.tra', '_tmp.tra')
 
@@ -1779,12 +1906,14 @@ def _appendTrlFile(trlfile, drizfile):
     except Exception:
         pass
 
+
 def _timestamp(_process_name):
     """Create formatted time string recognizable by OPUS."""
     _prefix = time.strftime("%Y%j%H%M%S-I-----", time.localtime())
     _lenstr = 60 - len(_process_name)
     return _prefix + _process_name + (_lenstr * '-') + '\n'
 
+
 def _getTime():
     # Format time values for keywords IRAF-TLM, and DATE
     _ltime = time.localtime(time.time())
@@ -1805,6 +1934,7 @@ def _createWorkingDir(rootdir, input):
         os.mkdir(newdir)
     return newdir
 
+
 def _copyToNewWorkingDir(newdir, input):
     """ Copy input file and all related files necessary for processing to the new working directory.
 
@@ -1825,6 +1955,7 @@ def _copyToNewWorkingDir(newdir, input):
         for fname in glob.glob(rootname + '*'):
             shutil.copy(fname, os.path.join(newdir, fname))
 
+
 def _get_envvar_switch(envvar_name):
     # interpret envvar variable, if specified
     if envvar_name in os.environ:
@@ -1839,17 +1970,20 @@ def _get_envvar_switch(envvar_name):
 
     return switch_val
 
+
 def _restoreResults(newdir, origdir):
     """ Move (not copy) all files from newdir back to the original directory
     """
     for fname in glob.glob(os.path.join(newdir, '*')):
         shutil.move(fname, os.path.join(origdir, os.path.basename(fname)))
 
+
 def _removeWorkingDir(newdir):
     """ Delete working directory
     """
     os.rmdir(newdir)
 
+
 def rmtree2(path, n=3):
     """Wrapper around shutil.rmtree to make it more robust when used on NFS mounted file systems."""
     ok = False
@@ -1868,6 +2002,7 @@ def rmtree2(path, n=3):
     else:
         print("Path %s successfully removed." % path)
 
+
 def handle_remove_readonly(func, path, exc):
     excvalue = exc[1]
     if func in (os.rmdir, os.remove) and excvalue.errno == errno.EACCES:
@@ -1876,13 +2011,14 @@ def handle_remove_readonly(func, path, exc):
     else:
         raise
 
+
 # Functions to support execution from the shell.
 def main():
 
     import getopt
 
     try:
-        optlist, args = getopt.getopt(sys.argv[1:], 'bdahfgin:')
+        optlist, args = getopt.getopt(sys.argv[1:], 'bdahfgimn:v:')
     except getopt.error as e:
         print(str(e))
         print(__doc__)
@@ -1898,6 +2034,8 @@ def main():
     align_to_gaia = True
     debug = False
     force_alignment = False
+    do_verify_guiding = False
+    make_manifest = False
 
     # read options
     for opt, value in optlist:
@@ -1906,13 +2044,17 @@ def main():
         if opt == "-d":
             debug = True
         if opt == "-a":
-            force_alignment = False
+            force_alignment = True
         if opt == "-h":
             help = 1
         if opt == "-f":
             force = True
         if opt == "-i":
             inmemory = True
+        if opt == "-m":
+            make_manifest = True
+        if opt == "-v":
+            do_verify_guiding = True
         if opt == '-n':
             if not value.isdigit():
                 print('ERROR: num_cores value must be an integer!')
@@ -1922,7 +2064,7 @@ def main():
             # turn off writing headerlets
             headerlets = False
     if len(args) < 1:
-        print("syntax: runastrodriz.py [-fhibng] inputFilename [newpath]")
+        print("syntax: runastrodriz.py [-bdahfginv] inputFilename [newpath]")
         sys.exit()
     if len(args) > 1:
         newdir = args[-1]
@@ -1933,7 +2075,9 @@ def main():
         try:
             process(args[0], force=force, newpath=newdir, num_cores=num_cores,
                     inmemory=inmemory, headerlets=headerlets,
-                    align_to_gaia=align_to_gaia, force_alignment=force_alignment, debug=debug)
+                    align_to_gaia=align_to_gaia, force_alignment=force_alignment,
+                    do_verify_guiding=do_verify_guiding, debug=debug,
+                    make_manifest=make_manifest)
 
         except Exception as errorobj:
             print(str(errorobj))
